/*
IRTC, home of all raytracers
Paolo Gibellini
2009-05-25, from sratch

- random brick rotation
- Homes instead of bricks? Could be nice a irtc-city
- Grouped objects and rotated union
- Poser hand
- Rearranged with wings (in default poser export media doesen't worked)
- Centered and exported with poseray v3.12.0.452
- Rearrange camera

toDo: some people?
toDo: homes or...birds searching for bread?

Thanks:
Mike (http://povray.tashcorp.net/tutorials/qd_sierpinski/) for pyramid
Newsgroup for initial cloud texture
*/


#include "colors.inc"
#include "skies.inc" 

#declare show_irtc=0;			// show the text traced
#declare show_hand=1;         	// show base traced for the elevation of the bricks
#declare show_bricks=1;         // "bricks" traced over a text but with the elevation of a ground - requires show_hand=1

// Global settings ------------------------------------
global_settings {
	adc_bailout 0.003922
	ambient_light <1.0,1.0,1.0>
	assumed_gamma 1 //1.9
	hf_gray_16 off
	irid_wavelength <0.247059,0.176471,0.137255>
	max_intersections 64
	max_trace_level 10
	number_of_waves 10
	noise_generator 2
	charset ascii
}


// Camera & lights -------------
#declare camera_look_at = <-1.0,1.0,0> ;
#declare camera_location = <0,8,-14>;
//#declare camera_location = <0,50,0>; // Flight view
//#declare camera_location = <0,0,-40>; // Front

camera {
	//orthographic
	location camera_location
	angle 40
	look_at camera_look_at
}

// Light ----------------------------------
light_source {
	camera_location
	color rgb <1,1,1>
	parallel
	point_at camera_look_at
}
// ----------------------------------------

 
//Background
//background { color rgb<0.7490196,0.8235294,0.8352941>  }
// Sky ----------------------------------------------------
sky_sphere {
    pigment {
        gradient y
        pigment_map {
            [0.10 rgb <0.258, 0.258, 0.435> ]
            [0.25 P_Cloud2 scale 0.15 ]
        }
        scale 0.3
        translate 0.2*y
    }
    pigment {
    	color rgbf<SkyBlue.x,SkyBlue.y,SkyBlue.z,0.8>
    }
}

// Textures ------------------------------------------
// Cloud texture from newsgroup:
#declare m_cloud=material{
	texture {
		pigment {
			color rgbt <1,1,1,1> //.95
		}
	}
	interior {
		media {
			//method 1
			samples 5.0 //50
			//intervals 4
			//absorption <0.25,0.33,0.67>
			absorption .4
			//emission <0.3,0.1,0.125>/2
			emission .1
			scattering {
				1, rgbt<0.25,0.25,0.25,0.98>
				extinction 0.28
			}
			density {
				/*
				agate
				density_map {
					[0.5 rgb 0.3]
					[0.67 rgb 0]
					[0.7 rgb 2]
					[1 rgb 3]
				}
				*/
				granite
				density_map {
					[0.67 rgb 0]
					[0.7 rgb 2]
					[1 rgb 3]
				}
				//scale <75,50,50>
				scale 400
				//scale 200
			}
		}
	}
	scale 13
}

#declare SkinBody_= m_cloud;
#declare FingerNails_=m_cloud; 
// ---------------------------------------------------------------

// IRTC -----------------------------
#declare rotate_irtc=<0, -5, 0>;
#declare translate_irtc=<-5.5, 5, -2.0>; 
#declare type_irtc_txt=text { ttf "times.ttf", "IRTC", 1, 0 rotate 90*x scale 3 scale 2*y} 
#declare type_irtc=object {type_irtc_txt rotate rotate_irtc translate translate_irtc}
#if(show_irtc)
    object{type_irtc texture{pigment {color rgbt <1,1,1,0.6>}}}
#end

// Cloudy hand --------------------------------------
//Assembled object that is contained in right_hand_p4_close_POV_geom.inc
#if(show_hand)
	#include "type_irtc_hand_010.inc" //Geometry
	//Model assembly from the meshes
	#declare right_hand_=union {
		object{right_hand_FingerNails_  material{FingerNails_} hollow}
		object{right_hand_SkinBody_  material{SkinBody_} hollow}
		scale -x
		rotate 90*x
		rotate -90*y
		rotate -70*z
		translate <.3,-.35,-0.1>
		scale 13
	}

	object{
		right_hand_
	}
#end
//==================================================

// Pebbles --------------------------
#if(show_bricks)
	#declare R1=seed(2);
	#declare R2=seed(41);
	#declare R3=seed(12);
	#declare l_box=0.1;
	
	#declare the_brick=union{
		// Home
		difference{
			box{0 l_box}
			// Interior
			box{0.1 l_box*0.9}
			// Door
			box{<l_box/3,0,-0.1> <2*l_box/3,l_box/2-l_box/10,2*l_box/3>}
			// Upper front and rear windows
			box{<l_box/10,l_box/2+l_box/10,-0.1> <l_box/4+l_box/10,l_box/2+l_box/4+l_box/10,l_box*1.1>}
			box{<l_box-l_box/10,l_box/2+l_box/10,-0.1> <l_box-l_box/4-l_box/10,l_box/2+l_box/4+l_box/10,l_box*1.1>}
			// Lower rear windows
			box{<l_box/10,l_box/10+l_box/10,l_box-0.1> <l_box/4+l_box/10,l_box/10+l_box/4+l_box/10,l_box-2*l_box/3>}
			box{<l_box-l_box/10,l_box/10+l_box/10,l_box-0.1> <l_box-l_box/4-l_box/10,l_box/10+l_box/4+l_box/10,l_box-2*l_box/3>}
			// Lateral upper windows
			box{<-0.1,l_box/2+l_box/10,l_box/10> <l_box*1.1,l_box/2+l_box/4+l_box/10,l_box/4+l_box/10>}
			box{<-0.1,l_box/2+l_box/10,l_box-l_box/10> <l_box*1.1,l_box/2+l_box/4+l_box/10,l_box-l_box/4-l_box/10>}
			// Lateral rear windows
			box{<-0.1,l_box/10,l_box/10> <l_box*1.1,l_box/10+l_box/4+l_box/10,l_box/4+l_box/10>}
			box{<-0.1,l_box/10+l_box/10,l_box-l_box/10> <l_box*1.1,l_box/10+l_box/4+l_box/10,l_box-l_box/4-l_box/10>}
			texture{pigment {color rgb 1}}
		}
		// 1st floor
		box{<0,l_box/2-l_box/10,0> <l_box,l_box/2,l_box> texture{pigment {color rgb 1}}}
		// Deck
		difference{
		    box {< 1,1,1>, <-1,0,-1>}
		    plane { x-y,  -sqrt(2)/2 }
		    plane { -x-y, -sqrt(2)/2 }
		    plane { z-y,  -sqrt(2)/2 }
		    plane { -z-y, -sqrt(2)/2 }
		    scale l_box*1.1/2
		    translate <0.5,1,0.5>*l_box
		    texture{pigment {color rgb x}}
		}
		// Chimney
		box {<l_box-l_box/10,l_box,l_box-l_box/10> <l_box-l_box/10-l_box/10,l_box+l_box/3,l_box-l_box/10-l_box/10> texture{pigment {color rgb 1}}}
		box {<l_box-l_box/10+l_box/30,l_box+l_box/3,l_box-l_box/10+l_box/30> <l_box-l_box/10-l_box/10-l_box/30,l_box+l_box/3+l_box/30,l_box-l_box/10-l_box/10+l_box/30> texture{pigment {color rgb 1}}}
	}
	//todo: cutoff of some pieces
	
	#declare Normal=<0,0,0>;
	#declare Normal_sand=<0,0,0>;
	#declare m_min=min_extent(type_irtc);
	#declare m_max=max_extent(type_irtc);
	
	#declare m_steps=100; //100
	#declare r_stop=0.90;
	
	#local i=0;
	union {
		#while (i<=m_steps)
			#local j=0;
			#local m_x=m_min.x+(m_max.x-m_min.x)*i/m_steps;
			#while (j<=m_steps)
				#local m_z=m_min.z+(m_max.z-m_min.z)*j/m_steps;
				#if(rand(R1)<=r_stop)
					#declare Intersection=trace(type_irtc, <m_x,10,m_z>, -y, Normal);
					#declare Intersection_sand=trace(right_hand_, <m_x,10,m_z>, -y, Normal_sand);
					#if(vlength(Normal) > 0)
						#local correction=0.5+rand(R2)*2;
	    				#local Rotation=(rand(R3)-0.5)*40;
						//object{the_brick scale correction rotate Rotation translate <Intersection.x,Intersection_sand.y,Intersection.z> }
						object{the_brick scale correction rotate Rotation translate <Intersection_sand.x,Intersection_sand.y,Intersection_sand.z> }
					#end
				#end
				#local j=j+1;
			#end
			#local i=i+1;
		#end
	}
#end
